public class Employe implements java.io.Serializable {
    protected static double salaire_base = 1430.22;
    static final int _HOMME = 0;
    static final int _FEMME = 1;

    private int genre; 
    private String nom;
    private Cadre superieur;

    /** Constructeurs
     */
    protected Employe(String nom, int genre) {
	this.nom = nom;
	this.genre = genre;
    }

    public Employe(String nom, int genre, Cadre superieur) throws Exception{
	this(nom,genre);
	this.superieur = superieur;
	superieur.ajouteSubordonne(this);
    }
    
   /**
     * accesseur pour le nom
     */
    public String getNom() {
    	return this.nom;
    }
   
   /**
     * accesseur pour le genre
     */
    public int getGenre() {
    	return this.genre;
    }

    /**
     * accesseur pour le superieur
     */
    public Cadre getSuperieur() {
    	return this.superieur;
    }

    /**
     * Conversion en string pour l'affichage, cette fonction est appelée automatiquement lors de la conversion en String
     * @return le nom de l'employé avec la formule de politesse Mr ou Mme pour un cadre,  suivi de " membre du service de ..." si cet employé a un supérieur
     */
    public String toString() {
    	if (this.superieur == null) {
    	    return this.getNom();
    	} else {
    	    return this.getNom() + " membre du service de " + this.superieur.getNom();
    	}
    }

   
    /**
     * Teste l'égalité, deux employés sont identique si leur nom et leur genre sont identique
     * @param e : l'employé avec lequel tester
     * @return vrai ou faux
     */
    boolean equals(Employe e) {
    	return ((this.nom.equals(e.nom))&&(this.genre == e.genre));
    }

    /**
     * Calcul du salaire
     * @return le salaire de l'employé
     */
    public double getSalaire() {
	return salaire_base;
    }


    // Questions facultatives :

     /**
     * Teste si l'employé a pour  supérieur hiérarchique (direct ou indirect) l'employé donné en argument
     */
    public boolean aPourSuperieur(Employe e) {
	if (superieur == null) return false;
	if (superieur.equals(e)) return true;
	return superieur.aPourSuperieur(e);
    }

    /**
     * Change l'affectation de l'employé auprès du cadre en argument
     */
    public void changementAffectation(Cadre c) throws Exception{
	if (c.aPourSuperieur(this)) {
	    throw new Exception("Changement d'affectation impossible car "+c.getNom()+" a pour supérieur hiérarchique "+this.getNom());
	}
	if (this.superieur != null) this.superieur.enleveSubordonne(this);
	c.ajouteSubordonne(this);
	this.superieur = c; 
    }

    /** 
     * Affiche le service  avec l'indentation mis en argument et retourne le nombre de personnes du service
     */ 
    int afficheService(String ind) {
	String tiret="";;
	if (ind!=""){
	    tiret ="- ";
	} 
	System.out.println(ind+tiret+this.getNom()+ " (Salaire: "+ this.getSalaire()+")"); 
	return 1;
    }


    // Méthode utilisée par retireEmploye dans la classe Entreprise
    void retireDeEntreprise(Entreprise ent){
	ent.emp.remove(this.getNom());
	System.out.println(this+" a été retirée de l'entreprise.");
    }
}
